from __future__ import unicode_literals
# encoding: utf-8

from django.db import models


class CepManager(models.Manager):
    def get_or_create(self, codigo):
        try:
            return Cep.objects.get(codigo=codigo)
        except Cep.DoesNotExist:
            return Cep(codigo=codigo)


class Cep(models.Model):
    codigo = models.CharField(max_length=8, unique=True)
    valido = models.BooleanField(default=False)
    cidade = models.CharField(max_length=255)
    estado = models.CharField(max_length=2)
    logradouro = models.CharField(max_length=255)
    bairro = models.CharField(max_length=255)
    complemento = models.CharField(max_length=255)

    objects = CepManager()

    def __str__(self):
        return '{0}.{1}-{2}'.format(
            self.codigo[0:2],
            self.codigo[2:5],
            self.codigo[5:8])



# encoding: utf-8
import re


class Parser(object):
    def __init__(self, content):
        self.content = content.decode('iso-8859-1')
        self.labels = None
        self.contents = None

    def get_data(self):
        self.get_labels()
        self.get_contents()

        return self.format_data()

    def format_data(self):
        data = {}
        for label, value in zip(self.labels, self.contents):
            if 'localidade' in label:
                cidade, estado = value.split('/')
                data['estado'] = estado.strip()
                data['cidade'] = cidade.strip()
            elif 'logradouro' in label and ' - ' in value:
                logradouro, complemento = value.split(' - ', 1)
                data['logradouro'] = logradouro.strip()
                data['complemento'] = complemento.strip()
            else:
                data[label] = value.strip()

        return data

    def get_labels(self):
        matches = re.findall(r'<span class="resposta">(.*?)<\/span>',
                             self.content,
                             re.M | re.I | re.S)
        self.labels = [i.lower().strip(' :') for i in matches]

    def get_contents(self):
        matches = re.findall(r'<span class="respostadestaque">(.*?)<\/span>',
                             self.content,
                             re.M | re.I | re.S)
        self.contents = [re.sub(r'\s+', ' ', i) for i in matches]



# encoding: utf-8
from __future__ import unicode_literals

import codecs
import os
import mock
import sys
import requests

test_dir = os.path.dirname(os.path.abspath(__file__))
sys.path.insert(0, os.path.join(test_dir, os.path.pardir))
os.environ["DJANGO_SETTINGS_MODULE"] = 'tests.settings'

import django
from django.test import TestCase
from django.core.management import call_command
from django.core.exceptions import ValidationError
django.setup()
call_command('migrate', '--run-syncdb')

from cep.models import Cep
from cep.forms import CepField
from cep.parser import Parser


class FakeRequest(object):
    def __init__(self, content):
        self.content = content


class FakeException(requests.RequestException):
    pass


def fake_request_success_brasilia(*args, **kwargs):
    with codecs.open('tests/responses/success.html', 'r', 'iso-8859-1') as f:
        return FakeRequest(f.read().encode('iso-8859-1'))


def fake_request_success_logradouro(*args, **kwargs):
    with codecs.open('tests/responses/success-logradouro.html',
                     'r',
                     'iso-8859-1') as f:
        return FakeRequest(f.read().encode('iso-8859-1'))


def fake_request_fail(*args, **kwargs):
    with codecs.open('tests/responses/error.html', 'r', 'iso-8859-1') as f:
        return FakeRequest(f.read().encode('iso-8859-1'))


def fake_request_error(*args, **kwargs):
    raise FakeException('Internet Down')


class CepModelTestCase(TestCase):
    def test_can_isntantiate(self):
        cep = Cep(codigo='11111111')
        cep.save()
        self.assertIsInstance(cep, Cep)

    def test_sucessive_saves_increments_id(self):
        cep = Cep(codigo='11111111')
        cep.save()
        novo_cep = Cep(codigo='11111112')
        novo_cep.save()
        self.assertEqual(2, novo_cep.id)

    def test_cep_str(self):
        cep = Cep(codigo='11111111')
        cep.save()
        self.assertEqual('11.111-111', str(cep))


class CepFormTestCase(TestCase):
    def test_invalid_cep_format(self):
        field = CepField()
        with self.assertRaises(ValidationError):
            field.clean('701150-903')

    def test_invalid_cep_with_only_ones(self):
        field = CepField()
        with self.assertRaises(ValidationError):
            field.clean('111111111')

    @mock.patch('requests.post', mock.Mock(side_effect=fake_request_fail))
    def test_validate_with_correios_invalid_cep(self):
        field = CepField()
        with self.assertRaises(ValidationError):
            field.clean('71150-903')

    @mock.patch('requests.post',
                mock.Mock(side_effect=fake_request_success_brasilia))
    def test_correctly_cep(self):
        field = CepField()
        self.assertEqual('70.150-903',
                         field.clean('70.150-903').original_value)

    @mock.patch('requests.post',
                mock.Mock(side_effect=fake_request_success_brasilia))
    def test_correctly_cep_with_only_dash(self):
        field = CepField()
        self.assertEqual('70150903',
                         field.clean('70150-903').codigo)

    @mock.patch('requests.post', mock.Mock(side_effect=fake_request_error))
    def test_validate_without_internet_silent(self):
        field = CepField(force_correios_validation=False)
        self.assertEqual('70.150-903',
                         field.clean('70.150-903').original_value)

    @mock.patch('requests.post', mock.Mock(side_effect=fake_request_error))
    def test_validate_without_internet_raises_exception(self):
        field = CepField()
        with self.assertRaises(ValidationError):
            field.clean('70.150-903')

    @mock.patch('requests.post',
                mock.Mock(side_effect=iter(fake_request_success_brasilia,
                                           fake_request_error)))
    def test_revalidate_saved_cep(self):
        field = CepField()
        field.clean('70.150-903')
        self.assertEqual('70.150-903',
                         field.clean('70.150-903').original_value)

    @mock.patch('requests.post',
                mock.Mock(side_effect=fake_request_success_brasilia))
    def test_validate_fulfill_module(self):
        field = CepField()
        cep = field.clean('70.150-903')
        self.assertEqual('Zona Cívico-Administrativa', cep.bairro)

    @mock.patch('requests.post',
                mock.Mock(side_effect=fake_request_success_brasilia))
    def test_validate_fulfill_module_logradouro_with_client(self):
        field = CepField()
        cep = field.clean('70.150-903')
        self.assertEqual('Palácio da Alvorada (Residência'
                         ' Oficial do Presidente da República)',
                         cep.logradouro)

    @mock.patch('requests.post',
                mock.Mock(side_effect=fake_request_success_logradouro))
    def test_validate_fulfill_module_logradouro_without_client(self):
        field = CepField()
        cep = field.clean('70.150-903')
        self.assertEqual('Rua Doutor Raul Silva',
                         cep.logradouro)

    @mock.patch('requests.post',
                mock.Mock(side_effect=fake_request_success_logradouro))
    def test_validate_fulfill_module_logradouro_with_complemento(self):
        field = CepField()
        cep = field.clean('70.150-903')
        self.assertEqual('de 2301/2302 ao fim',
                         cep.complemento)

    def test_request_timeout(self):
        cep = CepField(timeout=0.0001)
        with self.assertRaises(ValidationError):
            cep.clean('70.150-903')

    def test_request_timeout_without_raise_error(self):
        cepfield = CepField(force_correios_validation=False, timeout=0.0001)
        cep = cepfield.clean('70.150-903')
        self.assertFalse(cep.valido)


class ParserTestCase(TestCase):
    def setUp(self):
        self.data = {
            'logradouro': 'Rua Doutor Raul Silva',
            'bairro': 'Jardim Francisco Fernandes',
            'cidade': 'São José do Rio Preto',
            'estado': 'SP',
            'complemento': 'de 2301/2302 ao fim',
        }

    def test_parser_gets_bairro(self):
        response = fake_request_success_logradouro()
        parser = Parser(response.content)
        parsed_data = parser.get_data()
        self.assertEqual(self.data['bairro'],
                         parsed_data['bairro'])

    def test_parser_gets_cidade(self):
        response = fake_request_success_logradouro()
        parser = Parser(response.content)
        parsed_data = parser.get_data()
        self.assertEqual(self.data['cidade'],
                         parsed_data['cidade'])

    def test_parser_gets_bairro_com_acento(self):
        data = {
            'logradouro': 'SPP',
            'bairro': 'Zona Cívico-Administrativa',
            'cidade': 'Brasília',
            'estado': 'DF',
        }
        response = fake_request_success_brasilia()
        parser = Parser(response.content)
        parsed_data = parser.get_data()
        self.assertEqual(data['cidade'],
                         parsed_data['cidade'])

    def test_parser_gets_estado(self):
        response = fake_request_success_logradouro()
        parser = Parser(response.content)
        parsed_data = parser.get_data()
        self.assertEqual(self.data['estado'],
                         parsed_data['estado'])

    def test_parser_gets_logradouro(self):
        response = fake_request_success_logradouro()
        parser = Parser(response.content)
        parsed_data = parser.get_data()
        self.assertEqual(self.data['logradouro'],
                         parsed_data['logradouro'])

    def test_parser_gets_complemento(self):
        response = fake_request_success_logradouro()
        parser = Parser(response.content)
        parsed_data = parser.get_data()
        self.assertEqual(self.data['complemento'],
                         parsed_data['complemento'])



# encoding: utf-8
from __future__ import unicode_literals
import requests

from django import forms
from django.core.exceptions import ValidationError

from .models import Cep
from .parser import Parser


class CepField(forms.RegexField):
    SERVICE_URL = 'http://m.correios.com.br/movel/buscaCepConfirma.do'

    def __init__(self, force_correios_validation=True, timeout=10, *args, **kwargs):
        super(CepField, self).__init__(r'^\d{2}\.?\d{3}-?\d{3}$',
                                       *args,
                                       **kwargs)
        self.force_correios_validation = force_correios_validation
        self.dados = {
            'bairro': None,
            'logradouro': None,
            'estado': None,
            'cidade': None,
            'cliente': None,
        }
        self.valido = False
        self.timeout = timeout

    def clean(self, value):
        original_value = value
        value = super(CepField, self).clean(value)
        value = value.replace('.', '').replace('-', '').strip(' :')
        cep = Cep.objects.get_or_create(codigo=value)
        cep.original_value = original_value
        if cep.valido:
            return cep

        if not self.valida_correios(value):
            raise ValidationError('Invalid CEP')

        cep.valido = self.valido
        if cep.valido:
            cep.logradouro = self.dados.get('logradouro',
                                            self.dados.get('cliente', ''))
            cep.bairro = self.dados.get('bairro', '')
            cep.estado = self.dados.get('estado', '')
            cep.cidade = self.dados.get('cidade', '')
            cep.complemento = self.dados.get('complemento', '')
        cep.save()
        return cep

    def valida_correios(self, codigo):
        try:
            response = requests.post(
                self.SERVICE_URL,
                {'metodo': 'buscarCep', 'cepEntrada': codigo},
                timeout=self.timeout)
            parser = Parser(response.content)
            self.dados = parser.get_data()
        except requests.RequestException:
            if self.force_correios_validation:
                raise ValidationError('Cannot validade with Correios')
            return True

        if 'Dados nao encontrados' in parser.content:
            return False
        self.valido = True
        return True



SECRET_KEY = "secret"
DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    },
}

INSTALLED_APPS = (
    'cep',
)

# Removendo migrations
class DisableMigrations(object):
    def __contains__(self, item):
        return True

    def __getitem__(self, item):
        return "notmigrations"

MIGRATION_MODULES = DisableMigrations()



from setuptools import setup, find_packages

__version__ = '0.1.5'
__description__ = 'Django CepField'
__long_description__ = 'Auto fill address using brasizilian CEP number.'

__author__ = 'Marcus Bodock'
__author_email__ = 'mbodock@gmail.com'

requires = [i.strip() for i in open("requirements.txt").readlines()]

testing_extras = [
    'pytest',
    'pytest-cov',
]

setup(
    name='django-cepfield',
    version=__version__,
    author=__author__,
    author_email=__author_email__,
    packages=find_packages(),
    license='MIT',
    description=__description__,
    long_description=__long_description__,
    url='https://github.com/mbodock/django-cepfield/',
    keywords='Django, CEP, Address, Brazil',
    include_package_data=True,
    zip_safe=False,
    install_requires=requires,
    classifiers=[
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Operating System :: OS Independent',
        'Topic :: Software Development',
        'Environment :: Web Environment',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'License :: OSI Approved :: MIT License',
    ],
    tests_require=['pytest'],
    extras_require={
        'testing': testing_extras,
    },
)
